const Twig = require('twig');
const fs = require('fs');
const path = require('path');
const util = require('util');
const less = require('less');
const directoryTree = require('directory-tree');
const readFile = util.promisify(fs.readFile);
const writeFile = util.promisify(fs.writeFile);
const twigRender = util.promisify(Twig.renderFile);

function Overview({ folder = '/', extensions, exclude, fileName = 'index', returnString = false } = {}) {
  let paths = {
    folder: path.normalize(folder),
    less: path.join(__dirname, '/style/style.less'),
    twig_page: path.join(__dirname, './template/index.twig'),
    twig_list_el: path.join(__dirname, './template/list-el.twig'),
  }

  function convertPathToFolderRelative(item) {
    if (paths.folder === '/') return;
    item.path = item.path.replace(paths.folder, '');
  }

  async function getPage() {
    try {
      let data = await Promise.all([
        getStyles(),
        getList(directoryTree(paths.folder, { extensions, exclude }, convertPathToFolderRelative).children),
      ]);
      let html = await twigRender(paths.twig_page, {
          less: data[0].css,
          list: data[1],
        }
      );
      return html;
    } catch (e) {
      console.error('Ошибка при формировании страницы', e);
      throw new Error(e);
    }
  }

  async function getStyles() {
    try {
      let less_raw = await readFile(paths.less, 'utf8');
      let css = await less.render(less_raw);
      return css;
    } catch (e) {
      console.error('Ошибка получения файла стилей', e);
      throw new Error(e);
    }
  }

  async function getList(arr) {
    try {
      arr.sort(sortFileAndFolders);
      arr = await Promise.all(arr.map(async (item) => {
        return await getListEl(item);
      }));
      return arr.join('');
    } catch (e) {
      console.error('Ошибка формирования списка файлов', e);
      throw new Error(e);
    }
  }

  function sortFileAndFolders(a, b) {
    if (a.type === 'directory' && b.type === 'directory') {
      return 0;
    } else if (a.type === 'directory') {
      return -1;
    } else if (b.type === 'directory') {
      return 1;
    } else {
      return 0;
    }
  }

  async function getListEl(el) {
    let list;
    if (el.children) list = await getList(el.children);
    let html = await twigRender(paths.twig_list_el, {
        folder: el.type === 'directory',
        name: el.name,
        children: list,
        link: el.path,
      }
    );
    return html;
  }

  function generateOutputFile(html) {
    writeFile(paths.folder + fileName + '.html', html, 'utf8').catch((e) => {
      console.error('Ошибка записи готового файла', e);
    });
  }

  getPage().then((html) => {
    returnString ? html : generateOutputFile(html);
  });
}

module.exports = Overview;

// TODO добавить тесты